#!/usr/bin/env node

var gitlog = require('gitlog');
var program = require('commander');
var colors = require('colors');
var pkg = require('./package.json');
 
program.version(pkg.version)
    .option('-l, --list', 'List the used scopes')
    .option('-c, --count', 'List the used scopes and their frequency')
    .option('-m, --max [limit]', 'Limits the number of commits to look though')
    .option('-a, --alphabetise', 'Orders the scopes in alpabetical order')
    .option('-f --frequency', 'Order counted list i order of most frequent');

program.on('--help', showHelp);
program.parse(process.argv);
  
var options = { 
    repo: '.', 
    number: (program.max && (program.max.toString() !== 'true')) ? program.max : 2000,
    fields:['body']
} 
 
if (program.list) {
    gitlog(options, function(error, commits) {
    
        // get commit body
        var commitMessages = getCommitBody(commits);
        
        // remove dupes
        commitMessages = commitMessages.filter(function(elem, pos) {
            return commitMessages.indexOf(elem) == pos;
        }); 
    
        // show scopes
        commitMessages.forEach(function(message) {
            console.log(colors.green(message));
        });
    });
    
}

if (program.count) {
    gitlog(options, function(error, commits) {
    
        // get commit body
        var commitMessages = getCommitBody(commits);
        
        // count scopes    
        var scopes = getScopesObject(commitMessages);
    
        if (program.alphabetise) {
            // show scopes in order
            alphabetiseScopeObject(scopes);
        } else {
            if (program.frequency) {
                // show scopes in frequency order
                frequencyScopeObject(scopes);  
            } else {
                // show scopes
                for (var key in scopes) {
                    console.log(
                        colors.green(scopes[key]) + ' ' + 
                        colors.white('x') + ' ' + 
                        colors.green(key)
                    );
                } 
            }
        }
    });
}


function getCommitBody(commits) {
    var messages = commits.map(function (commit) {
        return commit.body.split(/\(|\)/)[1];
    });
    
    // remove undefined
    messages = messages.filter(function(elem) {
        if (elem !== undefined) return elem;
    });
    
    if (program.alphabetise) {
        return messages.sort();
    }
    
    return messages
}

function getScopesObject (commitMessages) {
    var scopes = {};
    
    commitMessages.forEach(function(key) { 
        scopes[key] = (scopes[key] || 0)+1; 
    });
    
    return scopes;
}

function alphabetiseScopeObject(scopes) {
    var keys = [];
    
    for (var key in scopes) {
        keys.push(key);
    }
      
    keys = keys.sort();       
    
    keys.forEach(function(key) {
        console.log(
            colors.green(scopes[key]) + ' ' + 
            colors.white('x') + ' ' + 
            colors.green(key)
        );  
    });                            
}

function frequencyScopeObject(scopes) {
    var tmp = [];
    
    for (var key in scopes) {
        tmp.push([key, scopes[key]])
        
        tmp.sort(function(a, b) {
            return b[1] - a[1];
        })
    }
    
    tmp.forEach(function(scope) {
        console.log(
            colors.green(scope[0]) + ' ' + 
            colors.white('x') + ' ' + 
            colors.green(scope[1])
        );  
    }); 
}

function showHelp () {
    console.log(colors.green('Examples:'));
  
    console.log('');
    console.log(
        colors.white('    $') + ' ' + 
        colors.green('cmzs -l')+ ' ' + 
        colors.grey('// shows a list of all the used scopes')
    );

    console.log(
        colors.white('    $') + ' ' + 
        colors.green('cmzs -l -a')+ ' ' + 
        colors.grey('// alphabetically ordered list of all the used scopes')
    );
    
    console.log(
        colors.white('    $') + ' ' + 
        colors.green('cmzs -l -m 10')+ ' ' + 
        colors.grey('// Lists all the used scopes in the last 10 commits')
    );

    console.log(
        colors.white('    $') + ' ' + 
        colors.green('cmzs -c')+ ' ' + 
        colors.grey('// shows a list of all the used scopes and the frequency')
    );
    
    console.log(
        colors.white('    $') + ' ' + 
        colors.green('cmzs -c -m 10')+ ' ' + 
        colors.grey('// Lists scopes and frequency in the last 10 commits')
    );
    
    console.log(
        colors.white('    $') + ' ' + 
        colors.green('cmzs -c -f')+ ' ' + 
        colors.grey('// shows a list of all the used scopes ordered by frequency')
    );
    
    console.log(
        colors.white('    $') + ' ' + 
        colors.green('cmzs -c -a')+ ' ' + 
        colors.grey('// alphabetically ordered scopes and frequencies')
    );
}

if (!program.list && !program.count) {
    program.outputHelp(function (message) {
        return message;
    });
}

